//
//	ViewController.m
//

#import "ViewController.h"
#import "SampleAppDelegate.h"

@implementation ViewController

#pragma mark -
#pragma mark Initialization


- (id)init {
	if ((self = [super initWithStyle:UITableViewStyleGrouped])) {
	}
	return self;
}


#pragma mark -
#pragma mark View lifecycle


- (void)viewDidLoad {
	[super viewDidLoad];

	UILabel	*titleLabel;
	CGRect	frame = CGRectMake(0, 0, 200, 40);
	titleLabel = [[UILabel alloc] initWithFrame:frame];
	titleLabel.text = @"DB Test";
	titleLabel.font = [UIFont systemFontOfSize:16];
	titleLabel.textColor = [UIColor whiteColor];
	titleLabel.shadowColor = [UIColor darkGrayColor];
	titleLabel.shadowOffset = CGSizeMake(0, -1);
	titleLabel.backgroundColor = [UIColor colorWithWhite:0 alpha:0];
	titleLabel.textAlignment = UITextAlignmentCenter;
	titleLabel.adjustsFontSizeToFitWidth = YES;
	titleLabel.minimumFontSize = 9;
	self.navigationItem.titleView = titleLabel;
	[titleLabel release];

	//バーにボタンを追加
	UIBarButtonItem *configButton = [[UIBarButtonItem alloc]
									 initWithTitle:@"Test"
									 style:UIBarButtonItemStyleBordered
									 target:self
									 action:@selector(dbTest)];
	self.navigationItem.rightBarButtonItem = configButton;
	[configButton release];

	// スクロールのロック
	self.tableView.scrollEnabled = NO;
}

- (void)viewWillAppear:(BOOL)animated {
	[super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated {
	[super viewDidAppear:animated];
 	self.navigationController.toolbarHidden = YES;
}


#pragma mark -
#pragma mark Table view data source


- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	// Return the number of sections.
	return 1;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	// Return the number of rows in the section.
	return 0;
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {

	static NSString *CellIdentifier = @"Cell";

	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
	if (cell == nil) {
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
		cell.selectionStyle = UITableViewCellSelectionStyleNone;
	}

	return cell;
}


#pragma mark -
#pragma mark Memory management


- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
	[super didReceiveMemoryWarning];

	// Relinquish ownership any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Relinquish ownership of anything that can be recreated in viewDidLoad or on demand.
	// For example: self.myOutlet = nil;
}


- (void)dealloc {

	[super dealloc];
}


#pragma mark -
#pragma mark Database test method


-(void)dbTest {
	@try {
		// DB接続
		[self dbOpen];

		@try {
			// トランザクションの開始
			[self dbBegin];

			// Delete
			[self userInfoDeleteAll];

			// Prepare
			[self prepareInsertSQL];


			// Insert
			UserInfo* userInfo;
			userInfo = [UserInfo ID:@"111111"
							   Name:@"Name1"
						 ScreenName:@"ScreenName1"
					ProfileImageUrl:@"http://url1"
					 FollowersCount:0
					   FriendsCount:0
							   Text:@"Text1"];
			[self userInfoInsert:userInfo];
			[userInfo release];

			userInfo = [UserInfo ID:@"222222"
							   Name:@"Name2"
						 ScreenName:@"ScreenName2"
					ProfileImageUrl:@"http://url2"
					 FollowersCount:0
					   FriendsCount:0
							   Text:@"Text2"];
			[self userInfoInsert:userInfo];
			[userInfo release];

			// Finalize
			[self finalizeInsertSQL];

			// Select
			[self userInfoSelect];

			// DBの更新を確定する。
			[self dbCommit];
		} @catch (DBException* ex) {
			// DBエラーの処理を記述
			NSLog(@"%@ %@(%@)", [ex name], [ex message], [ex method]);
			// DBをもとの状態に戻す。
			[self dbRollback];
		}
		// DB切断
		[self dbClose];
		// catch節に、@catch (DBException* ex)と書くと、DBExceptionを捕まえてくれます。
	} @catch (DBException* ex) {
		// DB Openエラーの処理を記述
		NSLog(@"%@ %@(%@)", [ex name], [ex message], [ex method]);
		// DBExceptionじゃない場合は、次のcatch節で処理されます。
	} @catch (NSException* ex) {
		// その他エラーの処理を記述
		NSLog(@"%@/%@",[ex name], [ex reason]);
	} @finally {
		// 必要なら実装
	}
}


#pragma mark -
#pragma mark Database


-(void)dbOpen {
	NSLog(@"DB Open");

	NSString* database_filename;
	NSString* template_path;
	NSString* database_path;
	NSString* work_path;

	// データベース名
	database_filename = @"twitterDB.sqlite";

	// データベースファイルを格納するために文書フォルダーを取得します。
	work_path = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];

	// データベースファイルのパスを取得します。
	database_path = [NSString stringWithFormat:@"%@/%@", work_path, database_filename];

	// 文書フォルダーにデータベースファイルが存在しているかを確認します。
	NSFileManager* manager = [NSFileManager defaultManager];

	if (![manager fileExistsAtPath:database_path])
	{
		NSError* error = nil;

		// 文書フォルダーに存在しない場合は、データベースの複製元をバンドルから取得します。
		template_path = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:database_filename];

		// バンドルから取得したデータベースファイルを文書フォルダーにコピーします。
		if (![manager copyItemAtPath:template_path toPath:database_path error:&error])
		{
			// データベースファイルのコピーに失敗した場合の処理です。
			DBException* exception =
			[DBException exceptionWithCode:DB_OPEN_ERROR
									  Name:@"DB Error"
									Method:[NSString stringWithFormat:@"%@/%@",
											NSStringFromClass([self class]),
											NSStringFromSelector(_cmd)]];
			@throw exception;
		}
	}

	// 文書フォルダーに用意されたデータベースファイルを開きます。
	if (sqlite3_open([database_path UTF8String], &db_) != SQLITE_OK)
	{
		// データベースファイルを SQLite で開くことに失敗しました。
		DBException* exception =
		[DBException exceptionWithCode:DB_OPEN_ERROR
								  Name:@"DB Error"
								Method:[NSString stringWithFormat:@"%@/%@",
										NSStringFromClass([self class]),
										NSStringFromSelector(_cmd)]];
		@throw exception;
	}
}

-(void)dbClose {
	NSLog(@"DB Close");

	if (db_) {
		// DB Close
		sqlite3_close(db_);
	}
	db_ = nil;
}

-(void)dbBegin {
	NSLog(@"DB Transaction start");
	sqlite3_exec(db_, "BEGIN", NULL, NULL, NULL );
}

-(void)dbCommit {
	NSLog(@"DB Commit");
	sqlite3_exec(db_, "COMMIT", NULL, NULL, NULL );
}

-(void)dbRollback {
	NSLog(@"DB Rollback");
	sqlite3_exec(db_, "ROLLBACK", NULL, NULL, NULL );
}

-(void)userInfoInsert:(UserInfo*)userInfo {
	NSLog(@"DB Insert");
	sqlite3_reset(statementInsert_);
	sqlite3_bind_text  (statementInsert_, 1, [userInfo.ID UTF8String], -1, SQLITE_TRANSIENT);
	sqlite3_bind_text  (statementInsert_, 2, [userInfo.Name UTF8String], -1, SQLITE_TRANSIENT);
	sqlite3_bind_text  (statementInsert_, 3, [userInfo.ScreenName UTF8String], -1, SQLITE_TRANSIENT);
	sqlite3_bind_text  (statementInsert_, 4, [userInfo.ProfileImageUrl UTF8String], -1, SQLITE_TRANSIENT);
	sqlite3_bind_double(statementInsert_, 5, [userInfo.FollowersCount doubleValue]);
	sqlite3_bind_double(statementInsert_, 6, [userInfo.FriendsCount doubleValue]);
	sqlite3_bind_text  (statementInsert_, 7, [userInfo.Text UTF8String], -1, SQLITE_TRANSIENT);
	int wasSucceeded = sqlite3_step(statementInsert_);

	if( wasSucceeded != SQLITE_DONE ) {
		NSLog( @"Failed to insert from database with '%s'.", sqlite3_errmsg( db_ ));
		DBException* exception =
		[DBException exceptionWithCode:DB_INSERT_ERROR
								  Name:@"DB Error"
								Method:[NSString stringWithFormat:@"%@/%@",
										NSStringFromClass([self class]),
										NSStringFromSelector(_cmd)]];
		@throw exception;
	}
}

-(void)prepareInsertSQL {
	NSString* insertSQL  = @"insert into userinfo values (?, ?, ?, ?, ?, ?, ?)";

	if(sqlite3_prepare_v2(db_, [insertSQL UTF8String], -1, &statementInsert_, NULL) != SQLITE_OK)
	{
		NSLog( @"Failed to prepare statement with '%s'.", sqlite3_errmsg( db_ ));
		DBException* exception =
		[DBException exceptionWithCode:DB_PREPARE_ERROR
								  Name:@"DB Error"
								Method:[NSString stringWithFormat:@"%@/%@",
										NSStringFromClass([self class]),
										NSStringFromSelector(_cmd)]];
		@throw exception;

	}
}

-(void)finalizeInsertSQL {
	if (db_) {
		// PREPARE済みSTATEMENTの廃棄
		sqlite3_finalize(statementInsert_);
	}
}

-(void)userInfoSelect {
	NSLog(@"DB Select");

	UserInfo* userInfo;

	// 発行するSQL
	NSString* selectSQL = [NSString stringWithFormat:@"select * from userinfo"];

	// SQLのコンパイル
	sqlite3_stmt *statement = nil;
	if(sqlite3_prepare_v2(db_, [selectSQL UTF8String], -1, &statement, NULL) == SQLITE_OK) {
		int wasSucceeded = 0;

		// 次の行が存在する時、戻り値はSQLITE_ROWを返却する。ないとき、SQLITE_DONEを返却する。
		do
		{
			// SQLの実行
			wasSucceeded = sqlite3_step(statement);
			if (wasSucceeded == SQLITE_ROW)
			{
				userInfo = [UserInfo ID:[self getString:statement column:0]
								   Name:[self getString:statement column:1]
							 ScreenName:[self getString:statement column:2]
						ProfileImageUrl:[self getString:statement column:3]
						 FollowersCount:[self getNumber:statement column:4]
						   FriendsCount:[self getNumber:statement column:5]
								   Text:[self getString:statement column:6]];
				NSLog(@"%@", userInfo);
				[userInfo release];
			} else if(wasSucceeded == SQLITE_DONE) {
				//読み切り
			} else {
				//PREPARE済みSTATEMENTの廃棄
				sqlite3_finalize(statement);			}
		} while (wasSucceeded == SQLITE_ROW);
	} else {
		NSLog( @"Failed to select statement with '%s'.", sqlite3_errmsg( db_ ));
		DBException* exception =
		[DBException exceptionWithCode:DB_PREPARE_ERROR
								  Name:@"DB Error"
								Method:[NSString stringWithFormat:@"%@/%@",
										NSStringFromClass([self class]),
										NSStringFromSelector(_cmd)]];
		@throw exception;
	}
	// PREPARE済みSTATEMENTの廃棄
	sqlite3_finalize(statement);
}

// 値の取出し
-(NSNumber*)getNumber:(sqlite3_stmt*)statement column:(int)col {
	return [NSNumber numberWithInt:(int)sqlite3_column_int(statement, col)];
}

-(NSString*)getString:(sqlite3_stmt*)statement column:(int)col {
	char *work = (char *)sqlite3_column_text(statement, col);
	if (work == NULL) {
		return nil;
	} else {
		return [NSString stringWithUTF8String:work];
	}
}

-(void)userInfoDeleteAll {
	NSLog(@"DB Delete all");

	NSString *deleteSQL = @"DELETE FROM userInfo";

	// SQL文のコンパイルと実行
	sqlite3_stmt *statement = Nil;
	if( sqlite3_prepare_v2(db_, [deleteSQL UTF8String], -1, &statement, NULL ) != SQLITE_OK) {
		NSLog( @"Failed to prepare statement with '%s'.", sqlite3_errmsg( db_ ));
		DBException* exception =
		[DBException exceptionWithCode:DB_PREPARE_ERROR
								  Name:@"DB Error"
								Method:[NSString stringWithFormat:@"%@/%@",
										NSStringFromClass([self class]),
										NSStringFromSelector(_cmd)]];
		@throw exception;
	}
	int wasSucceeded = sqlite3_step(statement);

	//PREPARE済みSTATEMENTの廃棄
	sqlite3_finalize(statement);
	if( wasSucceeded != SQLITE_DONE ) {
		NSLog( @"Failed to delete from database with '%s'.", sqlite3_errmsg( db_ ));
		DBException* exception =
		[DBException exceptionWithCode:DB_DELETE_ERROR
								  Name:@"DB Error"
								Method:[NSString stringWithFormat:@"%@/%@",
										NSStringFromClass([self class]),
										NSStringFromSelector(_cmd)]];
		@throw exception;
	}
}

@end

